# = ajaxim daemon - ajax Instant Messenger server
#
# == Description
#
# This is a Ruby ajaxIM server which handles all requests from drupal client.
#
# == Authors
#
# * Aleksey Khodakovskiy <mailto:ask.monk@gmail.com>
# * Arthur Murauskas <mailto:arthur.murauskas@gmail.com>
#
# == Requirements
# === Gems
# * gem install mysql
# * gem install json
# * gem install daemons
#
# ===Apache modules
# * mod_proxy
# * mod_proxy_http
#
# == Configuration
#
# === Configuring apache
# Apache mod_proxy should be configured to redirect queries from <drupal_path/ajaxim_server> to <localhost:2000>.
# Here is example configuration of mod_proxy:
#  ProxyPass http://my_drupal/drupal/ajaxim_server http://localhost:2000/
#  ProxyPassReverse http://my_drupal/drupal/ajaxim_server http://localhost:2000/
#
# === Configuring daemon
# Set all daemon options in daemon.config.
# Example configuration:
# ----
#  # full path to directory where logs will be stored
#  log_dir: /home/valcker/projects/ajaxim/logs
#  
#  # logging is on(true) or off(false)
#  logs: true
#  
#  # interval of update status check in seconds
#  status_update_int: 180
#  
#  # database settings
#  db:
#    host: localhost
#    user: root
#    password: ''
#    db_name: drupal5
#  
#  http_port: 2000
# ----
#
# == Usage
#
# To use ajaxIM daemon you should run command(only Linux):
#  ruby ajaxim_daemon_controller.rb [start|stop|restart]
#
# This will run daemon in Windows and in interactive mode in Linux:
#  ruby ajaxim_daemon_controller.rb run
#
#
class AjaxIMDaemonController #:nodoc:
  require 'rubygems'         # if you use RubyGems
  require 'daemons'
  require 'lib/ajaxim_drupal_user.rb'
  require 'lib/ajaxim_logger.rb'
  require 'lib/ajaxim.rb'
  require 'lib/ajaxim_string.rb'
  require 'lib/ajaxim_drupaldb.rb'
  require 'lib/php_serialize.rb'
  require 'lib/ajaxim_config.rb'
  
  $config = AjaxIMConfig.new('./daemon.config')

  Daemons.run('lib/ajaxim_daemon.rb')
end
