require 'json'

class AjaxIM
  attr_reader :user

  def initialize(db, from, tid, recipient, message)
    @recipient = recipient
    @message = message
    @db = db
    @user = User.new(from, @db, tid)
    @buddies = Hash.new
    @@logger = AjaxIMLogger
    @@logger.set_path($config.log_dir)
    @logging = $config.logs
  end
  
  # sending a message
  # params is a hash of message parameters: font, etc
  # in this method @recipient can be either user name, or room name
  def send(params)
    # getting user status and checking if user is a chatroom
    user_status = user_online?(@recipient)
    is_room = is_chatroom?(@recipient)
    
    # if user is not a room and has status == 0 he is offline
    return 'not_online' unless user_status > 0 || is_room == true
    
    return 'too_long' if @message.strip.size > 1500
    return nil if @message.strip.size == 0

    message = <<-MESSAGE_STRING
      <span style="font-family:#{params['font']},sans-serif;font-size:#{params['font_size']}px;color:#{params['font_color']};">
      #{params['font_bold'] == true ? '<b>' : ''} #{params['font_underline'] == true ? '<u>' : ''} #{params['font_italic'] == true ? '<i>' : ''} #{@message} #{params['font_bold'] == true ? '</b>' : ''} #{params['font_underline'] == true ? '</u>' : ''} #{params['font_italic'] == true ? '</i>' : ''} </span>
    MESSAGE_STRING
    message = @db.escape_string(message)
    sql_to_insert = Array.new
    if is_room
      @@logger.room_message(@user['name'], @recipient, @message) if @logging == true
      # sending messages to all users in room
      # here @recipient is a room name
      get_chatlist(@recipient).each do |user_in_room|
        next if user_in_room['uid'] == @user['uid']
        id = @db.next_id("{ajaxim_messages}")
        sql_to_insert.push("(#{id}, '#{message}', 'msg', #{@user['uid']}, #{user_in_room['uid']}, '#{@recipient}')")
      end
    else
      recipient_user = User.new(@recipient, @db)
      #
      # when recipient is in 'friends_only' mode
      # and sender is not is recipients buddy list,
      # sender can't send him messages
      if user_status == 100
        return 'not_online' unless buddy_get_buddies(recipient_user['uid']).has_key?(@user['uid'])
      end
      @@logger.private_message(@user['name'], recipient_user['name'], @message) if @logging == true
      id = @db.next_id("{ajaxim_messages}")
      sql_to_insert.push("(#{id}, '#{message}', 'msg', #{@user['uid']}, #{recipient_user['uid']}, '')")
    end
    
    if !sql_to_insert.empty?
      @db.query("INSERT INTO {ajaxim_messages}
                 (id, message, type, sender, recipient, room)
                 VALUES #{sql_to_insert.join(",")}")
    end
    
    'sent'
  end
  
  # loading user status/online and offline buddies/list of the user chatrooms/list of available chatrooms by criteria
  def login
    # selecting status and updating status from the previous login
    sql = "SELECT status, last_status FROM {ajaxim_status}
           WHERE uid = '" + @db.escape_string(@user['uid']) + "'"
    res = @db.query(sql)
    if res.num_rows == 0
      sql = "INSERT INTO {ajaxim_status} (status, uid, last_ping, last_status, last_stmsg)
             VALUES
               ('1', '" + @user['uid'] + "', '" + Time.now.to_i.to_s + "', '1', 'I\\'m available')" 
      res = @db.query(sql)
      status = 1
    else
      row = res.fetch_hash
      if row['last_status'].to_i == 0
        set = 'status=1, last_status=1'
        status = 1
      else
        set = "status=#{row['last_status']}"
        status = row['last_status']
      end
      sql = "UPDATE {ajaxim_status} 
             SET #{set}
             WHERE uid = '" + @user['uid'] +"'"
      res = @db.query(sql)
    end
    #
    # getting online/offline buddies
    buddylist_array = get_buddies(@user['uid'])
    #
    # getting user's chatrooms
    users_chats_array = Hash.new
    user_chats = Array.new
    
    sql = "SELECT room FROM {ajaxim_chats} WHERE uid = '" + @user['uid'] + "'"
    res = @db.query(sql)
    while uchat = res.fetch_hash do
      user_chats.push(uchat["room"]) 
      users_chats_array[uchat["room"]] = Hash.new
    end
    #
    # getting available chatrooms by criteria
    chats_array = Array.new
    sql = "SELECT room, COUNT(*) as num FROM {ajaxim_chats} GROUP BY room"
    res = @db.query(sql)
    while chat = res.fetch_hash do
      if chat["num"].to_i > 0 && !user_chats.include?(chat["room"])
        #chats_array[chat["room"]] = Hash.new  
        chats_array.push(chat['room'])
      end
    end
    buddylist = user_event(buddylist_array, "status,#{status}");
    output = Hash.new
    output["buddy"] = buddylist.merge(users_chats_array).to_json
    output["rooms"] = chats_array.sort.to_json
    output.to_json
    #{"buddy":"{\"Buddies\":[\"user1\",\"user2\"]}"}
    #{"buddy":"{\"Buddies\":[{\"username\":\"user1\",\"is_online\":\"2\"},{\"username\":\"user2\",\"is_online\":\"0\"}]}"}
  end
  
  # verifing user status for recipient 
  def isuser
    sql = "SELECT status FROM {ajaxim_status} WHERE uid = '"+ @user["uid"]+"'"
    res = @db.query(sql)
    if res.num_rows > 0
      user_status_array = res.fetch_hash
      if user_status_array["status"] == 100
        recipient = User.new(@recipient, @db)
        recipient_buddies = buddy_get_buddies(recipient["uid"]);
        if recipient_buddies.has_key?(@user["uid"])
          user_status_array["status"]
        else
          '0'
        end
      else
        user_status_array["status"]
      end
    else
      'not_exists'
    end
  end
  
  # joining room
  # * room - the room to join
  def joinroom(room)
    #
    # checks if room name contains invalid symbols or empty
    return 'invalid_chars' if (room != room.sanitize_alphanum() || room.nil? || room.empty?)
    #
    # checks if users is already joined room
    sql = "SELECT id FROM {ajaxim_chats} WHERE uid = '" + @user["uid"]+ "' AND room = '" + room + "'"
    res = @db.query(sql)
    return 'already_joined' if res.num_rows > 0 
    #
    # checks if room name is equal existing user name
    return 'room_is_user' if !User.new(room, @db).nil?
    #
    # writing to log user joinroom event
    @@logger.room_event(@user['name'], room, 'Joined room') if @logging == true
    
    id = @db.next_id("{ajaxim_chats}")
    sql = "INSERT INTO {ajaxim_chats} (id, room, uid) 
           VALUES
              ('" + id.to_s + "', '" + room + "', '" + @user["uid"].to_s+ "')"
    res = @db.query(sql)
    # select and notify chatmates about user's join 
    sql = "SELECT a.uid, u.name FROM {ajaxim_chats} a
           LEFT JOIN {users} u ON u.uid = a.uid
           WHERE a.room = '" + room + "' AND u.uid!='" + @user["uid"] + "'"
    res = @db.query(sql)
    output = {'users' => Array.new}
    res.each_hash do |user|
      output["users"].push(user["name"])
    end
    notify_buddies = user_event(output, "chat,join,#{room}");
    #
    # Do not show users in chatroom
    output['users'].clear
    #
    output.to_json
  end
  
  # leaving room
  # * room - the room to leave
  def leaveroom(room)
    # checks if room name contains invalid symbols or empty
    return 'invalid_chars' if (room != room.sanitize_alphanum() || room.nil? || room.empty?)
    #
    sql = "SELECT id FROM {ajaxim_chats} WHERE uid = '" + @user["uid"] + "' AND room = '" + room + "'"
    res = @db.query(sql)
    
    if res.num_rows > 0 
      # writing to log user leaveroom event
      @@logger.room_event(@user['name'], room, 'Left room') if @logging == true
      
      room_info = res.fetch_hash
      # leave room
      sql = "DELETE FROM {ajaxim_chats} WHERE id = '" + room_info["id"] + "'"
      @db.query(sql)
      #
      # notify chatmates about user's leaving
      sql = "SELECT a.uid, u.name FROM {ajaxim_chats} a 
             LEFT JOIN {users} u ON u.uid = a.uid 
             WHERE a.room = '" + room + "'"
      res = @db.query(sql)
      output = {'users' => Array.new}
      res.each_hash do |user|
        output['users'].push(user["name"])
      end
      notify_buddies = user_event(output, "chat,left,#{room}");

      'left'
    end
  end
  
  # ping the server
  # since the server cannot contact the client, the client must ping the server for new msgs
  #  * new_user_status - on ping user status
  #  * new_user_status_msg - on ping user status msg
  def ping(new_user_status, new_user_status_msg)
    # get last user status
    sql = "SELECT status FROM {ajaxim_status} WHERE uid = '"+ @user["uid"] +"'"
    res = @db.query(sql)
    #
    if res.num_rows > 0 
      ust = res.fetch_hash
      user_status = ust["status"]
      # get user's buddylist
      bl = buddy_get_buddies(@user["uid"])
      user_bl = {"Buddies" => Array.new}
      bl.each_pair do |buddy_uid, buddy|
        user_bl["Buddies"].push(buddy["name"])
      end
      #
      # set new status, status msg and last ping
      if user_status.to_i != 0
        sql = "UPDATE {ajaxim_status}
               SET status='#{new_user_status}',
                 last_ping='#{Time.now.to_i.to_s}',
                 last_status='#{new_user_status}',
                 last_stmsg='#{new_user_status_msg}'
               WHERE uid = '#{@user["uid"]}'"
        res = @db.query(sql)

        if user_status.to_i != new_user_status.to_i
          user_event(user_bl, 'status,'+new_user_status.to_s);
        end
        sql = "SELECT id, message, type, sender, recipient, room
               FROM {ajaxim_messages}
               WHERE recipient = '" + @user["uid"] + "'
               ORDER BY id ASC"
        res = @db.query(sql)
        to_delete = Array.new
        # initialization of the output array
        output = {
            'numMessages' => 0,
            'numEvents' => 0,
            'messages' => Array.new,
            'events' => Array.new}
        #
        res.each_hash do |message|
          sender = User.new(message["sender"].to_i, @db)
          # verification of the message type and pushing them into output array
          case message["type"] 
            when 'msg' :
              output["messages"].push({
                'message' => message["message"],
                'sender' => sender["name"],
                'recipient' => message["room"].length==0 ? @user["name"] : message["room"]})
              output["numMessages"]+=1
            when 'event' :
              output["events"].push({
                'event' => message["message"],
                'sender' => sender["name"],
                'group' => 'Buddies', # can be implemented multigroup buddylist
                'recipient' => @user["name"]})
              output["numEvents"]+=1
          end
          to_delete.push(message["id"])
        end
        # delete all "used" messages and events
        if to_delete.length > 0
          sql = "DELETE FROM {ajaxim_messages} WHERE id IN (" + to_delete.join(",") + ")"
          @db.query(sql)
        end
        #
        output.to_json
      end
      #
    else
      sql = "INSERT INTO {ajaxim_status} SET uid = '" + @user["uid"] + "', status = '0', last_ping = '" + Time.now.to_i.to_s + "'"
      @db.query(sql)
      'not_logged_in';
    end
  end
  
  def pingroom
    sql = "SELECT room FROM {ajaxim_chats} WHERE uid = '" + @user['uid'] + "'"
    res = @db.query(sql)
    user_chats = Array.new
    while uchat = res.fetch_hash do
      user_chats.push(uchat["room"]) 
    end
    #
    # getting available chatrooms by criteria
    criteria_users_num = get_variable("ajaxim_client_chatrooms_user_number" , 10)
    chats = Array.new
    sql = "SELECT room, COUNT(*) as num 
           FROM {ajaxim_chats} 
           WHERE uid != #{@user['uid']} 
           GROUP BY room"
    res = @db.query(sql)
    while chat = res.fetch_hash do
      if chat["num"].to_i >= criteria_users_num.to_i && !user_chats.include?(chat["room"])
        chats.push(chat['room'])
      end
    end
    

    output = {"rooms" => chats.sort.to_json}
    output.to_json
  end
 
  def update_status(interval)
    last_ping = (Time.new - interval).to_i.to_s
    res = @db.query("SELECT s.uid, u.name
                     FROM {ajaxim_status} s 
                     LEFT JOIN {users} u ON u.uid=s.uid
                     WHERE s.status > 0 AND s.last_ping < #{last_ping}")
    users = Array.new
    res.each_hash do |user|
#      user = {'uid' => row['uid'], 'name' => row['name']}
      buddy_list = get_buddies(user['uid'])
      user_event(buddy_list, 'status,0', user)
#      user_event(buddy_list, "chat,left,#{user['room']}", user)
      users.push(user['uid'])
    end
    @db.query("UPDATE {ajaxim_status}
               SET status=0 
               WHERE status>0 AND last_ping < #{last_ping}")
    @db.query("DELETE FROM {ajaxim_chats} 
               WHERE uid IN (#{users.join(',')})") unless users.empty?
  end
  
  private

  # performing user events functions
  #   * buddylist_n array of buddies
  #   * event event
  def user_event(buddylist_n, event, user=@user)
    sql_to_insert = Array.new
    event_info = event.split(",")
    return Hash.new unless buddylist_n.length > 0
    
    # verifing type of buddylist_n and parsing it to array
    if buddylist_n.kind_of?(String)
      buddylist = JSON.parse buddylist_n
    else
      buddylist = buddylist_n
    end
    
    # performing actions for different event types
    case event_info[0]
      when 'chat'
        users = buddylist["users"]
        sql = "SELECT aj.uid, u.name, aj.status
               FROM {ajaxim_status} aj
               LEFT JOIN {users} u 
                  ON u.uid=aj.uid 
               WHERE u.name IN ('#{users.join("','")}') AND u.name!='#{user["name"]}' AND aj.status > 0 
               ORDER BY u.name ASC"
        res = @db.query(sql)
        res.each_hash do |chatmate|
          id = @db.next_id("{ajaxim_messages}")
          sql_to_insert.push("(#{id}, '#{@db.escape_string(event)}', 'event', #{user["uid"]}, #{chatmate["uid"]})")
        end
      when 'status'
        # set of all my buddies
        sql = "SELECT b.buddy AS uid, u.name, aj.status 
               FROM {buddylist} b
               LEFT JOIN {users} u ON u.uid=b.buddy
               LEFT JOIN {ajaxim_status} aj ON aj.uid=b.buddy
               WHERE b.uid=#{user['uid']}"
        res = @db.query(sql)
        my_buddies = Array.new
        res.each_hash do |buddy|
          my_buddies.push({
            'uid'    => buddy['uid'],
            'name'   => buddy['name'],
            'status' => buddy['status']
          })
        end
        
        # set of all who have me in the list
        sql = "SELECT b.uid AS uid, u.name, aj.status
               FROM {buddylist} b
               LEFT JOIN {users} u ON u.uid=b.uid
               LEFT JOIN {ajaxim_status} aj ON aj.uid=b.uid
               WHERE b.buddy=#{user['uid']}"
        res.free
        res = @db.query(sql)
        i_am_buddy_of = Array.new
        res.each_hash do |buddy|
          i_am_buddy_of.push({
            'uid'    => buddy['uid'],
            'name'   => buddy['name'],
            'status' => buddy['status']
          })
        end
        
        event_buddies = Array.new
        if event_info[1].to_i == 100
          # TRUE buddies are those who are in our list and who have us in their lists
          # Thats why here we are selecting those, for whom we will be invisible:
          # they have us in list, but we don't
          event = "status,0"
          i_am_buddy_of.each do |buddy|
            event_buddies.push(buddy) unless my_buddies.include?(buddy)
          end
        else
          (i_am_buddy_of - my_buddies).each do |my_buddy|
            event_buddies.push(my_buddy)
          end
        end
        
        # generating sql for event notification
        event_buddies.each do |buddy|
          next if buddy['status'].to_i == 0
          id = @db.next_id('{ajaxim_messages}')
          sql_to_insert.push("(#{id}, '#{@db.escape_string(event)}', 'event', #{user["uid"]}, #{buddy['uid']})")
        end
        
        buddylist_new = Hash.new
        # forming array of buddies that will be sent to client
        buddylist.each_pair do |group, users|
          buddylist_new[group] = Array.new
          my_buddies.each do |my_buddy|
            if users.include?(my_buddy['name'])
              # checking if I am buddy of my_buddy
              # if not and he is invisible, then I shouldn't see him
              if my_buddy['status'].to_i == 100 && !i_am_buddy_of.include?(my_buddy)
                my_buddy['status'] = 0
              end
              buddylist_new[group].push({
                'username' => my_buddy['name'],
                'is_online' => my_buddy['status'].nil? ? 0 : my_buddy['status']
              })
            end
          end
        end
    end
    
    # sending messages to our friends and chatmates
    if sql_to_insert.length > 0
      sql = "INSERT INTO {ajaxim_messages} 
             (id, message, type, sender, recipient) 
             VALUES #{sql_to_insert.join(",")}"
      @db.query(sql)
    end
    
    buddylist_new
  end

  # ruby rewrite of drupal buddylist function
  def buddy_get_buddies(uid)
    if !@buddies.include?(uid)
      @buddies[uid] = Hash.new
      sql = 'SELECT b.buddy, u.name, u.mail, u.uid FROM {buddylist} b
            INNER JOIN {users} u ON b.buddy = u.uid
            WHERE b.uid = ' + uid.to_s
      res = @db.query(sql)
      res.each_hash do |row|
        @buddies[uid][row['buddy']] = {
          'uid' => row['uid'],
          'name' => row['name'],
          'mail' => row['mail']
        }
      end
      res.free
    end
    
    @buddies[uid]
  end
  
  # returns buddies array
  # Arguments: int uid - user id
  def get_buddies(uid)
    return Array.new if !get_variable("ajaxim_client_buddy_list", 1)
    buddies = buddy_get_buddies(uid)
    buddylist = {"Buddies" => Array.new}
    buddies.each_pair do |key, arr|
      buddylist['Buddies'].push(arr['name'])
    end
    
    buddylist
  end
  
  # rerurns settings variable value
  # Arguments: string name - variable name
  #            var default - variable default value
  def get_variable(name, default)
    sql = "SELECT value FROM {variable} WHERE name = '#{name}'"
    res = @db.query(sql)
    if res.num_rows > 0
      PHP.unserialize(res.fetch_hash["value"])
    else 
      default
    end
  end
  
  # Checks if a user is online or not
  # Returns user status, which can be:
  #   0   - there is no such user in table
  #   1   - available
  #   2   - busy, away, etc
  #   50  - invisible
  #   100 - friends only
  def user_online?(user_name)
    user_name = @db.escape_string(user_name.to_s)
    res = @db.query("SELECT a.status
                     FROM {ajaxim_status} a
                     LEFT JOIN {users} u ON u.name='#{user_name}'
                     WHERE a.uid=u.uid")
    user_online = res.num_rows > 0 ? res.fetch_row[0].to_i : 0
  end
  
  # checks whether room is a chatroom
  def is_chatroom?(room)
    room = @db.escape_string(room).downcase
    res = @db.query("SELECT room FROM {ajaxim_chats} c WHERE c.room='#{room}'")
    res.num_rows > 0
  end
  
  # Gets list of users in the room
  # Returns array of hashes {'name' => user name, 'uid' => user id}
  def get_chatlist(room)
    room = @db.escape_string(room).downcase
    res = @db.query("SELECT DISTINCT uid FROM {ajaxim_chats} c WHERE c.room='#{room}'")
    userlist = Array.new
    res.each_hash do |row|
      user = User.new(row['uid'].to_i, @db)
      userlist.push(user.user)
    end
    res.free
    userlist
  end
end
