# Main daemon class that creates daemon itself
class AjaxIMDaemon
  require 'webrick'
  include WEBrick

  @last_status_check = Time.new
  # status update interval in seconds
  check_interval = $config.status_update_int.to_i
  
  @db = DrupalDB.real_connect($config.db['host'],
                              $config.db['user'],
                              $config.db['password'],
                              $config.db['db_name'])
  @db.reconnect = true
  @s = HTTPServer.new( :Port => $config.http_port)
  @s.mount_proc("/"){|req, res|
      @request_method = req.request_method.to_s
      @request_path = req.path.to_s
      @call = req.query['call'].nil? ? '' : req.query['call'].sanitize_alphanum()
      @from = req.query['from'].nil? ? '' : req.query['from'].sanitize_alphanum()
      @tid = req.query['tid'].nil? ? '' : req.query['tid'].sanitize_alphanum()
      @recipient = req.query['recipient'].nil? ? '' : req.query['recipient'].sanitize_alphanum()
      @message = req.query['msg'].nil? ? '' : req.query['msg'].gsub("<amp>", "&").gsub(/<\/?[^>]*>/, "")
      # additional params
      # message parameters
      font           = req.query['font'].nil? ? '' : @db.escape_string(req.query['font'])
      font_size      = req.query['fontsize'].to_i > 24 ? 24 : req.query['fontsize'].to_i
      font_color     = req.query['fontcolor'].nil? ? '' : @db.escape_string(req.query['fontcolor'])
      font_bold      = req.query['bold'] == 'true' ? true : false
      font_italic    = req.query['italic'] == 'true' ? true : false
      font_underline = req.query['underline'] == 'true' ? true : false
      # room
      room = req.query['room']
      # user status 
      user_status = req.query['away'].nil? ? 0 : req.query['away'].to_i + 1
      user_status_msg = req.query['status'].nil? ? '' : @db.escape_string(req.query['status'])
      @ajaxIM = AjaxIM.new(@db, @from, @tid, @recipient, @message)

      
      # first we need to update ajaxim status table
      if (Time.new - @last_status_check) > check_interval
        @ajaxIM.update_status(check_interval)
        @last_status_check = Time.new
      end
      
      if @request_path != '/' || @ajaxIM.user.nil? == true # || @request_method!='POST' 
        res.body = 'not_logged_in'
      elsif
        res.body = case @call
          when 'login' : @ajaxIM.login
          when 'send' : @ajaxIM.send({
            'font'           => font,
            'font_size'      => font_size,
            'font_color'     => font_color,
            'font_bold'      => font_bold,
            'font_italic'    => font_italic,
            'font_underline' => font_underline
          })
          when 'room'      : @ajaxIM.pingroom
          when 'ping'      : @ajaxIM.ping(user_status, user_status_msg)
          when 'logout'    : 'logout' #backport - will be deleted later
          when 'save'      : 'saved' # backport - will be deleted later
          when 'isuser'    : @ajaxIM.isuser
          when 'joinroom'  : @ajaxIM.joinroom(room)
          when 'leaveroom' : @ajaxIM.leaveroom(room)
          else 'not_logged_in'
        end
      end
      res['Content-Type'] = "text/html"
  }
  trap("TERM"){ @s.shutdown }
  # gives ability to interrupt daemon from konsole
  trap("INT"){ @s.shutdown }
  @s.start
end
