=begin rdoc
  Logger class for ajaxIM daemon
=end

require 'singleton'

class AjaxIMLogger
  include Singleton

  def self.set_path(log_path)
    @@path = log_path
  end
  
  # logs private message
  def self.private_message(from, to, message)  
    # generating log filename
    fname = [from.strip, to.strip].sort.join('-')

    # formating log message
    str = format_message(from, message)

    # writing message
    write_message(fname, str, 'private')
  end

  # logs room message
  def self.room_message(from, room, message)
    # formating log message
    str = format_message(from, message)

    write_message(room.strip, str, 'room')
  end
  
  # logs room event
  def self.room_event(from, room, event_message)
    # formating log message
    str = format_message(from, event_message.strip.center(80, '*'))

    write_message(room.strip, str, 'room')
  end

  private
  
  # checks if all necessary directories are created
  # if not creates them
  def self.prepare_dir(dir)
    time = Time.new.strftime('%Y-%m-%d')
    #
    # checking for existing directory tree
    Dir.chdir(@@path)
    Dir.mkdir(time) if Dir[time].empty?
    Dir.mkdir("#{time}/#{dir}") if Dir["#{time}/#{dir}"].empty?
    Dir.chdir("#{time}/#{dir}")
  end
  
  def self.format_time
    Time.now.strftime('%H:%I:%S').ljust(15)
  end
  
  def self.format_message(from, message)
    str = format_time
    str += "<#{from.strip}>".ljust(20)
    str += message.strip
  end
  
  # writes message to file
  def self.write_message(fname, message, msg_type)
    prepare_dir(msg_type)
    fd = File.new(fname, 'a+')
    fd.puts(message)
    fd.close
  end
end