require "test/unit"
require 'rubygems'
require '../lib/ajaxim_logger.rb'
require '../lib/ajaxim_drupaldb.rb'
require '../lib/ajaxim_string.rb'
require '../lib/ajaxim.rb'
require '../lib/ajaxim_drupal_user.rb'
require '../lib/php_serialize.rb'
require '../lib/ajaxim_config.rb'

#
# Test unit for AjaxIM class
class TestAjaxIM < Test::Unit::TestCase
  $config = AjaxIMConfig.new('../daemon.config')
  
  def setup
    @db = DrupalDB.real_connect('localhost', 'root', '', 'drupal5')
    super
  end
  
  #
  # Tests login method of AjaxIM class
  def test_login
    params = [
      {
        'from'      => 'admin',
        'recipient' => '',
        'message'   => ''
      },
      {
        'from'      => 'valcker',
        'recipient' => '',
        'message'   => ''
      },
      {
        'from'      => "asdf'asf._--;,dad09354",
        'recipient' => '',
        'message'   => ''
      },
      {
        'from'      => 'valcker',
        'recipient' => 'lja.,m345"asdfm--__;;',
        'message'   => 'lja.,m345"asdfm--__;;'
      }
    ]
    
    msg 'Testing login'
    params.each do |param|
      puts "Params: "
      p param
      ajaxIM = AjaxIM.new(@db, param['from'], param['recipient'], param['message'])
      
      # we are checking this in ajaxim_daemon, so we don't need such check here
      next if ajaxIM.user.nil?
      
      responce = ajaxIM.login
      puts "Got responce from server: #{responce}"
      responce = JSON.parse responce
      
      assert_kind_of(Hash, responce, "Responce must be hash")
      assert(responce.has_key?('buddy'), "Responce hash doesn't have 'buddy' key")
      assert_kind_of(Hash, responce['buddy'], 'Buddy must be hash')
    end
  end
  
  def test_joinroom
    params = [
      {
        'from'      => 'admin',
        'recipient' => '',
        'message'   => '',
        'room'      => '',
        'responce'  => 'invalid_chars'
      },
      {
        'from'      => 'admin',
        'recipient' => '',
        'message'   => '',
        'room'      => 'valcker',
        'responce'  => 'room_is_user'
      },
      {
        'from'      => 'admin',
        'recipient' => '',
        'message'   => '',
        'room'      => 'ka7e54;a.sdf--a\'asdf"',
        'responce'  => 'invalid_chars'
      }
    ]
    
    msg 'Testing joinroom'
    #
    # testing room_is_user and invalid_chars checks
    params.each do |param|
      print "Params: "
      p param
      ajaxIM = AjaxIM.new(@db, param['from'], param['recipient'], param['message'])
      responce = ajaxIM.joinroom(param['room'])
      p "Responce: #{responce}"
      assert_match(param['responce'], responce)
    end
    #
    # testing user already_join check
    id = @db.next_id("{ajaxim_chats}")
    @db.query("INSERT INTO {ajaxim_chats} SET uid=1, room='test', id=#{id}")
    ajaxIM = AjaxIM.new(@db, 'admin', '', '')
    ajaxIM_valcker = AjaxIM.new(@db, 'valcker', '', '')
    responce = ajaxIM.joinroom('test')
    assert_match('already_joined', responce)
    responce = ajaxIM_valcker.joinroom('test')
    responce = JSON.parse responce
    assert(responce.has_key?('users') && responce['users'] == nil, 'responce must be hash and have users array == nil')
  end
  
  def test_send
    msg 'Testing send'
    # here we need next params:
    # from, recipient, message, and font params:
    # font, font_size, font_color, font_bold, font_italic, font_underline
    
    # testing not_online case: recipient is not online and is not a room
    from = 'admin'
    recipient = 'valcker'
    ajaxIM = AjaxIM.new(@db, from, recipient, '')
    responce = ajaxIM.send(Hash.new)
    assert_match('not_online', responce, 'User must be offline')
    
    # testing case when message.size == 0
    @db.query("INSERT INTO {ajaxim_status} SET status=100, uid=1, last_ping=#{Time.now.to_i.to_s}, last_status=0")
    from = 'valcker'
    recipient = 'admin'
    ajaxIM = AjaxIM.new(@db, from, recipient, '')
    responce = ajaxIM.send(Hash.new)
    assert(responce.nil?, 'Responce should be nil for empty messages')
    #
    # testing too_long case: message.size > 1500
    ajaxIM = AjaxIM.new(@db, from, recipient, 'a'*1501)
    responce = ajaxIM.send(Hash.new)
    assert_match('too_long', responce, 'Responce should be too_long for messages > 1500')
    #
    # testing for 'sent' return value when everything is ok
    ajaxIM = AjaxIM.new(@db, from, recipient, 'Hello world')
    responce = ajaxIM.send(Hash.new)
    assert_match('sent', responce, 'All params are OK, responce should be "sent"')
    #
    # testing for correct message sending
    params = {
      'font'           => 'tahoma',
      'font_size'      => 20,
      'font_color'     => 'black',
      'font_bold'      => false,
      'font_italic'    => false,
      'font_underline' => true
    }
    ajaxIM = AjaxIM.new(@db, from, recipient, 'Hi there')
    responce = ajaxIM.send(params)
    puts responce
    res = @db.query('SELECT * FROM {ajaxim_messages} ORDER BY id DESC').fetch_hash
    puts res['message']
  end
  
  def test_ping
    msg 'Testing ping'
    from = 'admin'
    recipient = 'valcker'
    @db.query("INSERT INTO {ajaxim_status} SET status=50, uid=1, last_ping=#{Time.now.to_i.to_s}, last_status=0")
    ajaxIM = AjaxIM.new(@db, from, recipient, '')
    responce = ajaxIM.ping('50', 'bla-bla')
    puts responce
  end
  
  def teardown
    @db.query('DELETE FROM {ajaxim_status}')
    @db.query("DELETE FROM {ajaxim_chats}")
    @db.query("DELETE FROM {ajaxim_messages}")
    @db.close
    super
  end
  
  private
  def msg(message)
    puts message.center(70, '*')
  end
end